# Transformacin y manipulacin de la base de datos Ozono 
# Troposfrico (O3)

# Libreras
suppressMessages(suppressWarnings(library(plyr)))
suppressMessages(suppressWarnings(library(dplyr)))
suppressMessages(suppressWarnings(library(readxl)))
suppressMessages(suppressWarnings(library(lubridate)))
suppressMessages(suppressWarnings(library(ggplot2)))
suppressMessages(suppressWarnings(library(reshape2)))

setwd("C:\\Users\\DELL\\Desktop\\TESIS_NMR\\Insumos Tesis NMR\\Datos Crudos v1")

# Cargamos las bases de datos del contaminante O3
# Observaciones del ao 2017
O3 <- read_excel("2017O3.xls") 
# Observaciones del ao 2016  
O3_2016 <- read_excel("2016O3.xls")

# Se tienen 8760 observaciones de los niveles de ozono 
# estratosfrico de 37 estaciones y las variables: hora,
# fecha y mes, en que fueron tomadas las mediciones.

etiquetas <- function(base) {

  # Formato das y meses
  dia <- format(base$FECHA, format="%a"); 
  mes <- month(as.POSIXlt(base$FECHA, format="%d/%m/%Y"))
  base <- data.frame(dia, mes, base) 
  
  # Etiquetas meses
  for(i in 1:length(base$mes)){
    base$mes[i][which(base$mes[i]==1)] <- "ENE"
    base$mes[i][which(base$mes[i]==2)] <- "FEB"
    base$mes[i][which(base$mes[i]==3)] <- "MAR"
    base$mes[i][which(base$mes[i]==4)] <- "ABR"
    base$mes[i][which(base$mes[i]==5)] <- "MAY"
    base$mes[i][which(base$mes[i]==6)] <- "JUN"
    base$mes[i][which(base$mes[i]==7)] <- "JUL"
    base$mes[i][which(base$mes[i]==8)] <- "AGO"
    base$mes[i][which(base$mes[i]==9)] <- "SEP"
    base$mes[i][which(base$mes[i]==10)] <- "OCT"
    base$mes[i][which(base$mes[i]==11)] <- "NOV"
    base$mes[i][which(base$mes[i]==12)] <- "DIC"
  }

  # Transformando variables
  base$FECHA <- as.Date(base$FECHA)
  base$HORA <- as.integer(base$HORA)
  base$mes <- as.factor(base$mes)
  base$mes <- factor(base$mes, levels=c("ENE", "FEB", "MAR", 
                                    "ABR", "MAY", "JUN",
                                    "JUL", "AGO", "SEP",
                                    "OCT", "NOV", "DIC"))
  base$dia <- as.factor(base$dia)
  base$dia <- factor(base$dia, levels=c("lun", "mar", "mi", 
                                    "jue", "vie", "sb", 
                                    "dom"))

  # Se modifica el valor de los valores faltantes de la 
  # base (-99) por NA's.

  for(i in 4:dim(base)[2]){
    base[,i][which(base[,i]==-99)]<-NA
  }
   return(base)
}

O3 <- etiquetas(O3)
# Estructura de la base de datos
str(O3)
summary(O3[,-c(1:4)])

# Se aplica la funcin "melt( )" de la librera "reshape2" 
# del software estadstico R a nuestra base de datos para
# tener una observacin por cada variable y poder manipularla
# con mayor facilidad. 

melt_O3 = melt(O3, id = c("dia","mes","FECHA", "HORA"))
colnames(melt_O3) <- c("dia","mes", "fecha", "hora",
                       "estacion", "concentracionO3" )
str(melt_O3)

# Frecuencia de observaciones que sobrepasan el valor permisible por hora

melt_O3 <- data.frame(cont=rep(1, dim(melt_O3)[2]), melt_O3)
mayor_95ppb <- melt_O3%>%
  group_by(cont, estacion)%>%
  filter(concentracionO3>95)%>%
  summarize(mayores=sum(cont))

# Concentraciones Diarias: Mximos de las concentraciones horarias 

melt_O3_max <- melt_O3%>%
  group_by(cont, fecha, dia, mes, estacion)%>%
  summarize(maximos=max(concentracionO3, na.rm = T))

melt_O3_max <- arrange(melt_O3_max, melt_O3_max$estacion)

#write.csv(melt_O3_max, file = "C:\\Users\\DELL\\Desktop\\TESIS_NMR\\Insumos Tesis NMR\\Datos Transformados v2\\O3_max_diarios_ma.csv")

# ---------------------------------------------------------------------------------------------------

# Grficos Estaciones de Monitoreo 
# Unidad de medicin de las observaciones: Promedio de la concentraciones del contaminante 
# O3 registradas en una hora (concentracin horaria), cuya unidad de medicin se denomina como 
# partes por billn.

theme_set(theme_classic())
theme_update(plot.title = element_text(hjust = 0.5, size = 24, face = "bold"), 
             plot.subtitle=element_text(size=20, hjust=0.5),
             plot.caption = element_text(size = 18),
             axis.title=element_text(size=24),
             axis.text = element_text(size=22),
             legend.text=element_text(size=24), 
             legend.position="bottom",
             legend.box = "horizontal")#centrar el titulo en las grficas 


# Histograma de la distribucin del promedio de las concentraciones de ozono registradas en una hr.

ggplot(melt_O3, aes(x=concentracionO3)) + geom_histogram(col="lightskyblue",fill="mediumblue", binwidth=2) +
  labs(title=" ", caption="Datos Red Automtica de Monitoreo Atmosfrico (RAMA)", 
       x=expression(O[3](ppb)), y=expression(Frecuencia)) + theme(panel.background = element_rect(fill = "white", colour = "grey50")) +
  scale_x_continuous(limits=c(0, 180)) +
  geom_vline(aes(xintercept = mean(concentracionO3, na.rm = T), color="media"),size=1) +
  geom_vline(aes(xintercept = median(concentracionO3, na.rm = T), color="mediana"),size=1) +
  scale_color_manual(name = "", values = c(media = "orange", mediana = "red"))

summary(melt_O3$concentracionO3)

# Boxplot concentracin horaria de ozono por estacin de monitoreo
ggplot(melt_O3, aes(x=estacion, y=concentracionO3)) + geom_boxplot(col="mediumblue") +
  labs(title=" ", y=expression(O[3](ppb)), x="Estacin", caption="Datos Red Automtica de Monitoreo Atmosfrico (RAMA)") +
  stat_summary(fun.y=mean, geom="point", shape=20, size=3, color="red", fill="red")+
  stat_summary(fun.y=median, geom="point", shape=20, size=3, color="blue", fill="blue") + theme(panel.background = element_rect(fill = "white", colour = "grey50")) + 
  theme(axis.text.x = element_text(angle = 90, hjust = 1)) +
  geom_hline(yintercept = 95, linetype="dashed", 
             color = "darkred", size=0.8)

# Frecuencia del nmero de horas en las que se super el valor 95ppb
mayor_95ppb <- melt_O3%>%
  filter(concentracionO3>95)

by_hora_mayor_95ppb <- as.data.frame(table(mayor_95ppb$hora))
by_dia_mayor_95ppb <- as.data.frame(table(mayor_95ppb$dia))
by_mes_mayor_95ppb <- as.data.frame(table(mayor_95ppb$mes))
by_estacion_mayor_95ppb <- as.data.frame(table(mayor_95ppb$estacion))

ggplot(by_estacion_mayor_95ppb, aes(x=Var1, y=Freq)) + geom_col(col="lightskyblue",fill="mediumblue") +
  labs(title=" ", x="Estacin", y="Frecuencia (horas)", caption="Datos Red Automtica de Monitoreo Atmosfrico (RAMA)") +
  guides(fill=guide_legend(title="")) + theme(panel.background = element_rect(fill = "white", colour = "grey50")) +
  theme(axis.text.x = element_text(angle = 90, hjust = 1)) +
  geom_hline(yintercept = max(by_estacion_mayor_95ppb$Freq), linetype="dashed", 
             color = "darkred", size=0.8)

# Boxplot de las concentraciones de ozono como promedio horario por mes de monitoreo 

melt_O3$mes <- as.factor(melt_O3$mes)
melt_O3$mes <- factor(melt_O3$mes, levels=c("ENE", "FEB", "MAR", 
                                    "ABR", "MAY", "JUN",
                                    "JUL", "AGO", "SEP",
                                    "OCT", "NOV", "DIC"))

ggplot(melt_O3, aes(x=mes, y=concentracionO3)) + geom_boxplot(col="mediumblue", fill="lightskyblue") +
  labs(title=" ", subtitle=" ", 
       caption="\nDatos Red Automtica de Monitoreo Atmosfrico (RAMA)", x="Mes", y=expression(O[3](ppb))) + 
  theme(panel.background = element_rect(fill = "white", colour = "grey50"))+
  stat_summary(fun.y=mean, geom="point", shape=20, size=3, color="red", fill="red")+
  stat_summary(fun.y=median, geom="point", shape=20, size=3, color="blue", fill="blue") + theme(panel.background = element_rect(fill = "white", colour = "grey50")) + 
  theme(axis.text.x = element_text(angle = 90, hjust = 1))


