# Modelo Polinomial

# Libreras
suppressMessages(suppressWarnings(library(MARSS)))
suppressMessages(suppressWarnings(library(imputeTS)))
suppressMessages(suppressWarnings(library(ggplot2)))
suppressMessages(suppressWarnings(library(forecast)))
suppressMessages(suppressWarnings(library(GGally)))
suppressMessages(suppressWarnings(library(changepoint))) 
suppressMessages(suppressWarnings(library(strucchange)))
suppressMessages(suppressWarnings(library(ggpmisc)))
suppressMessages(suppressWarnings(library(ggfortify)))
suppressMessages(suppressWarnings(library(lubridate)))
suppressMessages(suppressWarnings(library(readxl)))
suppressMessages(suppressWarnings(library(nortest)))

setwd("C:\\Users\\DELL\\Desktop\\TESIS_NMR\\Insumos Tesis NMR\\Datos Finales v3")

base <- read.csv("Datos_v3.csv")
attach(base)
fecha <- as.Date(as.character(fecha), "%d/%m/%Y")

# Diseo Grficas  
theme_set(theme_classic())
theme_update(plot.title = element_text(hjust = 0.5, size = 24, face = "bold"), 
             plot.subtitle=element_text(size=20, hjust=0.5),
             plot.caption = element_text(size = 18),
             axis.title=element_text(size=24),
             axis.text = element_text(size=22),
             legend.text=element_text(size=24), 
             legend.position="bottom",
             legend.box = "horizontal")#centrar el titulo en las grficas 

# Nmero de das de los datos 
TT = length(fecha)

# Variable respuesta Yt
Yt = matrix(Yt.s,nrow=1)

#--- Modelo de Polionomial de Orden 1 ---

# Definicin de los parmetros de la ecuacin del sistema

B = matrix(1)              ## Gt es igual a la constante 1
U = matrix(0)              ## escalar 0; 1x1
Q = matrix(list("Q"),1,1)  ## Matriz de varianzas del sistema;

# Definicin de los parmetros de la ecuacin de observaciones
Z = matrix(1)              ## Z=Ft'=1; escalar 1
A = matrix(0)              ## escalar 0; 1x1
R = matrix("r")            ## escalar r; 1x1; 

# Valores iniciales 
inits.list = list(x0=matrix(c(0), nrow=1))
# Lista de los parmetros que definen el modelo
mod.list = list(B=B, U=U, Q=Q, Z=Z, A=A, R=R)
# Ajuste del modelo polinomial de primer grado
poly = MARSS(Yt, inits=inits.list, model=mod.list)

cat("Estimador mximo verosimil de W:", poly$par$Q,
    "\nEstimador mximo versosimil de V:", poly$par$R,
    "\nSignal to noise ratio:", poly$par$Q/poly$par$R)

#--- Prediccin del modelo polinimial de grado 1 ---

# Salida del filtro de Kalman
kf.out = MARSSkfss(poly)
# Prnostico a un paso 
eta = kf.out$xtt1
# Media de la distribucin predictiva un paso adelante para los estados 
# at=E(thetha_1:t-1|Yt-1)=Gt*mt-1=mt-1
fore.mean = vector()
for(t in 1:TT) {
  fore.mean[t] = eta[,t,drop=F]
}


# Varianza un paso adelante de los parmetros de regresin al tiempo t; 1x2xT
Phi = kf.out$Vtt1
# Varianza de la ec. de observacin; matriz 1x1 
R.est = coef(poly, type="matrix")$R
# Varianza de la distribucin predictiva
# Rt=Var(thetha_1:t-1|Yt-1)=Gt*Ct-1*Gt'+Wt=Ct-1+Wt
fore.var = vector()
for(t in 1:TT) {
  fore.var[t] = Phi[,,t]+ R.est
}

fk_poly = fore.mean
fk_lower = fore.mean-2*sqrt(fore.var)
fk_upper = fore.mean+2*sqrt(fore.var)
forecast_poly <- data.frame(fecha=fecha, Obs=Yt.s, fk1=fore.mean, low=fk_lower, upp=fk_upper)
names(forecast_poly) <- c("fecha", "Yt.s", "fkpoly", "lower", "upper")

ggplot(forecast_poly, aes(x=fecha)) +
  geom_line(aes(y=Yt.s, colour="darkblue"), size=1.3) +
  #geom_point(aes(y=Yt.s, colour="red"), size=1.3) +
  geom_line(aes(y=fk_poly, colour="cyan"), size=1.3) + 
  geom_line(aes(y=lower, colour="cyan4"), size=1, linetype=3) +
  geom_line(aes(y=upper, colour="cyan4"), size=1, linetype=3) +
  scale_colour_manual("", values = c("cyan", "cyan4", "darkblue", "cyan4"), 
                      labels = c("Valores Predichos Modelo Polinomial, n=1", 
                                 "Intervalos al 95% de confianza", "Yt")) +
  labs(title=" ", x=" ", y=expression(O[3](ppb)),
       caption="\nDatos Red Automtica de Monitoreo Atmosfrico (RAMA)") +
  theme(panel.background = element_rect(fill = "white", colour = "grey50")) +
  scale_y_continuous(breaks=seq(-50, 250, 50)) #Marcas del -50 al 250, cada 50.


#--- Modelo Polinomial 2 ---

poly2 <- poly
poly2$par$Q <- 462

#--- Prediccin del modelo polinomial de grado 1, Wt=462 ---

# Salida del filtro de Kalman
kf.out2 = MARSSkfss(poly2)
# Prnostico a un paso 
eta2 = kf.out2$xtt1
# Media de la distribucin predictiva un paso adelante para los estados 
# at=E(thetha_1:t-1|Yt-1)=Gt*mt-1=mt-1
fore.mean2 = vector()
for(t in 1:TT) {
  fore.mean2[t] = eta2[,t,drop=F]
}


# Varianza un paso adelante de los parmetros de regresin al tiempo t; 1x2xT
Phi2 = kf.out2$Vtt1
# Varianza de la ec. de observacin; matriz 1x1 
R.est2 = coef(poly2, type="matrix")$R
# Varianza de la distribucin predictiva
# Rt=Var(thetha_1:t-1|Yt-1)=Gt*Ct-1*Gt'+Wt=Ct-1+Wt
fore.var2 = vector()
for(t in 1:TT) {
  fore.var2[t] = Phi2[,,t]+ R.est2
}

fk_poly2 = fore.mean2
fk_lower2 = fore.mean2-2*sqrt(fore.var2)
fk_upper2 = fore.mean2+2*sqrt(fore.var2)
forecast_poly2 <- data.frame(fecha=fecha, Obs=Yt.s, fk1=fore.mean2, low=fk_lower2, upp=fk_upper2)
names(forecast_poly2) <- c("fecha", "Yt.s", "fkpoly2", "lower2", "upper2")

ggplot(forecast_poly2, aes(x=fecha)) +
  geom_line(aes(y=Yt.s, colour="darkblue"), size=1.3) +
  #geom_point(aes(y=Yt.s, colour="red"), size=1.3) +
  geom_line(aes(y=fk_poly2, colour="palevioletred1"), size=1.3) + 
  geom_line(aes(y=lower2, colour="palevioletred3"), size=1, linetype=3) +
  geom_line(aes(y=upper2, colour="palevioletred3"), size=1, linetype=3) +
  scale_colour_manual("", values = c(  "darkblue","palevioletred1", "palevioletred3", "palevioletred3"), 
                      labels = c("Yt",
                                 "Valores Predichos Modelo Polinomial, n=1",
                                 "Intervalos de 95% de confianza" )) +
  labs(title=" ", x=" ", y=expression(O[3](ppb)),
       caption="\nDatos Red Automtica de Monitoreo Atmosfrico (RAMA)") +
  theme(panel.background = element_rect(fill = "white", colour = "grey50")) +
 scale_y_continuous(breaks=seq(0, 200, 50)) #Marcas del 0 al 70, cada 10.

#--- Diagnsticos para los residuales del modelo 1 --- 

# Errores de prediccion o innovaciones del modelo
innov1 = kf.out$Innov

# ACF de los residuales
acf(t(innov1), main="")
title(main="ACF de los Residuales", col.main="darkred")

# Ho: La media de la distribucin de las innovaciones es igual a cero 
# H1: La media de la distribucin de las innovaciones es distinta de cero

t.test(t(innov1))

# Ljung Box
Box.test(t(innov1), lag=20, type = "Ljung")
p_values_LB1 <- sapply(1 : 15, function(i)
  Box.test(t(innov1), lag=i, type="Ljung-Box")$p.value)

plot(p_values_LB1, type="p",  pch=19, ylim=c(0,1), xlim=c(0,16), lwd=1, col="blue", main=" ", xlab="Lag", ylab="p-valor")
abline(h=0.05, col="darkred", lty=3, lwd=2)
title(main="p-valores para el estadstico Ljung-Box", col.main="darkred")

# Histograma de los Residuales
x=(-35:35)/10
hist((t(innov1)-mean(t(innov1)))/sd(t(innov1)),nclass=50,freq=F,
     border="blue", col="skyblue", xlab="Residuales", 
     ylab="Densidad", main=" ")
title(main="Distribucin de los residuales", col.main="darkred")
lines(x,dnorm(x), lwd=2)

# Q-Q plot de los residuales
qqnorm(t(innov1), main=" ", ylab="Cuantiles muestrales", xlab="Cuantiles teoricos")
qqline(t(innov1), col="blue", lwd=1.8, lty=1.3)
title(main="Normal QQ-Plot", col.main="darkred")

# Prueba de Normalidad Anderson Darling
ad.test(t(innov1))
t.test(t(innov1))

#--- Diagnsticos para los residuales del modelo 2 --- 

# Errores de prediccion o innovaciones del modelo
innov2 = kf.out2$Innov

# ACF de los residuales
acf(t(innov2), main="")
title(main="ACF de los Residuales", col.main="darkred")

# Ho: La media de la distribucin de las innovaciones es igual a cero 
# H1: La media de la distribucin de las innovaciones es distinta de cero

t.test(t(innov2))

# Ljung Box
Box.test(t(innov2), lag=20, type = "Ljung")
p_values_LB2 <- sapply(1 : 15, function(i)
  Box.test(t(innov2), lag=i, type="Ljung-Box")$p.value)

plot(p_values_LB2, type="p",  pch=19, ylim=c(0,1), xlim=c(0,16), lwd=1, col="blue", main=" ", xlab="Lag", ylab="p-valor")
abline(h=0.05, col="darkred", lty=3, lwd=2)
title(main="p-valores para el estadstico Ljung-Box", col.main="darkred")

# Histograma de los Residuales
x=(-35:35)/10
hist((t(innov2)-mean(t(innov2)))/sd(t(innov2)),nclass=50,freq=F,
     border="blue", col="skyblue", xlab="Residuales", 
     ylab="Densidad", main=" ")
title(main="Distribucin de los residuales", col.main="darkred")
lines(x,dnorm(x), lwd=2)

# Q-Q plot de los residuales
qqnorm(t(innov2), main=" ", ylab="Cuantiles muestrales", xlab="Cuantiles teoricos")
qqline(t(innov2), col="blue", lwd=1.8, lty=1.3)
title(main="Normal QQ-Plot", col.main="darkred")

# Prueba de Normalidad Anderson Darling
ad.test(t(innov2))
t.test(t(innov2))
