# Regresin esttica y seleccin de variables

# Libreras
  suppressMessages(suppressWarnings(library(imputeTS)))
  suppressMessages(suppressWarnings(library(ggplot2)))
  suppressMessages(suppressWarnings(library(forecast)))
  suppressMessages(suppressWarnings(library(GGally))) 
  suppressMessages(suppressWarnings(library(changepoint))) 
  suppressMessages(suppressWarnings(library(strucchange)))
  suppressMessages(suppressWarnings(library(ggpmisc)))
  suppressMessages(suppressWarnings(library(ggfortify)))
  suppressMessages(suppressWarnings(library(lubridate)))
  suppressMessages(suppressWarnings(library(nortest))) 
  suppressMessages(suppressWarnings(library(lmtest)))
# Seleccin de variables
  suppressMessages(suppressWarnings(library(leaps)))
  suppressMessages(suppressWarnings(library(MASS)))
  suppressMessages(suppressWarnings(library(car)))
  suppressMessages(suppressWarnings(library(ggvis)))

  setwd("C:\\Users\\DELL\\Desktop\\TESIS_NMR\\Insumos Tesis NMR\\Datos Transformados v2")
  getwd()

# Base de las concentraciones diarias de ozono 
#  y las covariables: TMP, RH y WSP.

  base <- read.csv("O3_TMP_PED.csv") 
  fecha <- as.Date(as.character(base$fecha), "%d/%m/%Y")
  attach(base)

  str(base)
  summary(base)

  o3 = O3_max
  tmp = TMP_max
  rh = RH_max
  wsp = WSP_max

# Varianzas
  sapply(base[2:5], function(x){var(x,na.rm = T)})

# Proporcin valores faltantes
  sapply(base[2:5], function(x){length(which(is.na(x)))})
  sapply(base[2:5], function(x){length(which(is.na(x)))/length(x)})
  
  theme_set(theme_classic())
  theme_update(plot.title=element_text(hjust = 0.5, 
             size = 24, face="bold"), 
             plot.subtitle=element_text(size=19, hjust=0.5),
             plot.caption=element_text(size = 22),
             axis.title=element_text(size=22),
             axis.text=element_text(size=18),
             legend.text=element_text(size=18), 
             legend.position="bottom",
             legend.box="horizontal")

  # Scatterplot
  ggpairs(data.frame(o3, tmp, rh, wsp), 
        columnLabels = c("O3", "TMP", "RH", "WSP"),
        axisLabels = "show")

# Imputar valores faltantes
  Yt = na.kalman(base$O3_max)
  xt = na.kalman(base$TMP_max)
  x2t = na.kalman(base$RH_max)
  x3t = na.kalman(base$WSP_max)

  base.sin.na <- cbind(Yt, xt, x2t, x3t)

# Conversin de cada una de las variables en objetos de series de tiempo
  Yt.s <- ts(Yt, start = c(2017,1), frequency = 365)#concentraciones de O3
  xt.s <- ts(xt, start = c(2017,1), frequency = 365)#covariable temperatura
  x2t.s <- ts(x2t, start = c(2017,1), frequency = 365)#covariable humedad relativa
  x3t.s <- ts(x3t, start = c(2017,1), frequency = 365)#covariable WSP

# Estandarizar covariables
  xt.s <- ts((xt - mean(xt))/sqrt(var(xt)), 
              start = c(2017,1), frequency = 365)
  x2t.s <- ts((x2t - mean(x2t))/sqrt(var(x2t)), 
               start = c(2017,1), frequency = 365)
  x3t.s <- ts((x3t - mean(x3t))/sqrt(var(x3t)), 
               start = c(2017,1), frequency = 365)

  fecha <- as.character(base$fecha)
  base.s <- as.data.frame(cbind(Yt.s, xt.s, x2t.s, x3t.s))
  base.s <- cbind(fecha, base.s)
  summary(base.s)

  # Base de datos a emplear para el modelo predictivo  
  write.csv(base.s, file="C:\\Users\\DELL\\Desktop\\TESIS_NMR\\Insumos Tesis NMR\\Datos Finales v3\\Datos_v3.csv")
  
# Serie de tiempo O3
  ggplot(Yt.s, as.numeric = FALSE) +
  geom_line(col="blue", size=1.3) + 
  #  geom_point(col="red", size=1.8) +
  labs(title=" ",
       x="Tiempo", y=expression(O[3](ppb)),
       caption="\nDatos Red Automtica de Monitoreo Atmosfrico (RAMA)")  +
  theme(panel.background = element_rect(fill = "white", colour = "grey50"))
    
# Modelo esttico
  
  x1 <- xt.s
  x2 <- x2t.s
  x3 <- x3t.s  
  mod1 <- lm(Yt.s ~ x1 + x2 + x3)
  summary(mod1)

# Algoritmos de seleccin de variables

# Encuentra las mejores combinaciones con "p" predictores, i.e.,
# Para cada uno de los modelos generados
# (combinaciones con el mismo nmero de covariables t=1 o 2 o 3) 
# nos devuelve el "mejor" con base en el valor de rss 
# (suma de cuadrados de los residuales).  
  combinaciones <- regsubsets(Yt.s ~ x1 + x2 + x3, 
                            data=base.s, nbest = 7)

  summary(combinaciones)
  plot(combinaciones, scale="r2", xlab="Coeficientes de regresin")
 
  # Comparacin de los 3 mejores modelos
  
  #--- Validacin cruzada 
  
  #Elegiremos el mejor modelo sin importar el nmero de predictores p
  
  predict.regsubsets <- function(object, newdata, id ,...) {
    form <- as.formula(object$call[[2]]) 
    mat <- model.matrix(form, newdata)
    coefi <- coef(object, id = id)
    xvars <- names(coefi)
    mat[, xvars] %*% coefi
  }
  
  
  k <- 10
  set.seed(1)
  folds <- sample(1:k, nrow(base.s), replace = TRUE)
  table(folds)
  
  # n de obs en cada subconjunto de la muestra
  # 1  2  3  4  5  6  7  8  9 10 
  # 25 43 41 44 44 32 32 39 31 34
  
  #Creamos nuestra matriz de errores de prediccin 
  cv_errors <- matrix(NA, k, 3, dimnames = list(NULL, paste(1:3)))
  
  #Realizamos el mtodo de validacin cruzada de 10 iteraciones
  for(j in 1:k) {
    
    # Obtenemos nuestros mejores modelos del conjunto de entrenamiento
    best_subset <- regsubsets(Yt.s ~ xt.s+x2t.s+x3t.s, base.s[folds != j, ])
    
    # Mtodo de validacin cruzada
    for( i in 1:3) {
      #Valores predichos para los mejores modelos de p predictores con base en el conjunto de prueba
      pred_x <- predict.regsubsets(best_subset, base.s[folds == j, ], id = i)
      cv_errors[j, i] <- mean((base.s$Yt.s[folds == j] - pred_x)^2) #errores de prediccin
    }
  }
  
  # Obtenemos la media de los errores de prediccin, resultantes de las 10 iteraciones,
  # para cada uno de los 7 modelos

  mean.cv.errors = colMeans(cv_errors)
  
  #1        2        3 
  #588.7107 550.8936 549.9251
  
  which.min(mean.cv.errors)
  
  plot(mean.cv.errors, pch = 19, cex=1.5, type="b", col="blue", 
       xlab = "N de Covariables", ylab = "Media de los Errores de Prediccin")
  points(3, mean.cv.errors[3], pch = 20, cex=1.5, col = "red")
  
  # El modelo que minimiza el error es el modelo 7 -  Yt.s ~ TMP + WSP + RH,
  # seguido del modelo 4 - Yt.s ~ TMP + WSP
  
  # Basados en el AIC
    step <- stepAIC(mod1, direction="both")
    step$anova # Muestra los resultados 
  

#--- Validaciones de los residuales de los 3 mejores modelos ---


# Estimacin de parmetros para los 3 mejores modelos 
  coef(object = combinaciones, id = 1)
  coef(object = combinaciones, id = 4)
  coef(object = combinaciones, id = 7)

# Modelo Completo: Yt.s ~ xt.s + x2t.s + x3t.s

# Resumen del modelo
  summary(mod1)
# Intervalos de confianza
  confint(mod1)
# ACF de los residuales
  acf(mod1$residuals, main="")
  title(main="ACF de los residuales", col.main="darkred",
      cex.main=1.5)
# Pruebas de normalidad de los residuales
  qqnorm(mod1$residuals, main="")
  title(main="Normal Q-Q Plot", col.main="darkred",
      cex.main=1.5)
  qqline(mod1$residuals, col="blue")
  shapiro.test(mod1$residuals)
  ad.test(mod1$residuals)
  x=(-30:30)/10
  hist((mod1$res-mean(mod1$res))/sd(mod1$res), nclass=50, 
     freq=F, border="blue", col="skyblue", xlab="Residuales",
     cex.axis=1.3, cex.lab=1.3, ylab="Densidad", main=" ")
  title(main="Distribucin de los residuales", col.main="darkred",
      cex.main=1.5)
  lines(x,dnorm(x), lwd=2)
# Prueba de homocedasticidad de los residuales
  bptest(mod1)
  gvlma(mod1)

# Modelo con 2 covariables: Yt.s ~ xt.s + x3t.s

  mod2 <- lm(Yt.s ~ xt.s + x3t.s)

# Resumen del modelo
  summary(mod2)
# Intervalos de confianza
  confint(mod2)
# ACF de los residuales
  acf(mod2$residuals, main="")
  title(main="ACF de los residuales", col.main="darkred")
# Ljung Box
  Box.test(mod2$residuals, lag=20, type = "Ljung")
  p_values_LB <- sapply(1 : 15, function(i)
    Box.test(mod2$residuals, lag=i, type="Ljung-Box")$p.value)
  
  plot(p_values_LB, type="p",  pch=19, ylim=c(0,1), xlim=c(0,16), lwd=1, col="blue", main=" ", xlab="Lag", ylab="p-valor")
  abline(h=0.05, col="darkred", lty=3, lwd=2)
  title(main="P-Valores para el estadstico Ljung-Box", col.main="darkred")
# Pruebas de normalidad de los residuales
  qqnorm(mod2$residuals,  main="", xlab = "", ylab = "")
  title(main="Normal Q-Q Plot", col.main="darkred",
        xlab = "Cuantiles teoricos", ylab = "Cuantiles Muestrales")
  qqline(mod2$residuals, col="blue")
  shapiro.test(mod2$residuals)
  ad.test(mod2$residuals)
  jarque.bera.test(mod2$residuals)
  x=(-30:30)/10
  hist((mod2$res-mean(mod2$res))/sd(mod2$res),nclass=50, 
        freq=F, border="blue", col="skyblue", xlab="Residuales", 
        ylab="Densidad", main=" ")
  title(main="Distribucin de los residuales\n", col.main="darkred")
  lines(x,dnorm(x), lwd=2)
# Prueba de homocedasticidad de los residuales
  bptest(mod2)
  gvlma(mod2)

# Modelo con 1 covariable: Yt.s ~ xt.s 

  mod3 <- lm(Yt.s ~ xt.s)

# Resumen del modelo
  summary(mod3)
# Intervalos de confianza
  confint(mod3)
# ACF de los residuales 
  acf(mod3$residuals, main="")
  title(main="ACF de los residuales\n Modelo de regresin
        esttico una covariable", col.main="darkred",
        cex.main=1.5)
# Pruebas de normalidad de los residuales 
  qqnorm(mod3$residuals, main="")
  title(main="Normal Q-Q Plot\n Modelo de regresin lineal
        esttico una covariable", col.main="darkred",
        cex.main=1.5)
  qqline(mod3$residuals, col="blue")
  shapiro.test(mod3$residuals)
  jarque.bera.test(mod3$residuals)
  ad.test(mod3$residuals)
  x=(-30:30)/10
  hist((mod3$res-mean(mod3$res))/sd(mod3$res), nclass=50,
        freq=F, border="blue", col="skyblue", xlab="Residuales",
        cex.axis=1.3, cex.lab=1.3, ylab="Densidad", main=" ")
  title(main="Distribucin de los residuales", col.main="darkred",
        cex.main=1.5)
  lines(x,dnorm(x), lwd=2)
# Prueba de homocedasticidad de los residuales
  bptest(mod3)
  gvlma(mod3)

# Transformacin de Box - Cox

  boxCox(Yt.s ~ xt.s + x2t.s + x3t.s, 
         lambda=seq(-2,2,length=21), family="yjPower")
  title(main="Modelo completo", col.main="darkred",
        cex.main=1.5)
  boxCox(Yt.s ~ xt.s + x3t.s , lambda=seq(-2,2,length=21),
         family="yjPower")
  title(main="Modelo dos covariables", col.main="darkred",
        cex.main=1.5)
  boxCox(Yt.s ~ xt.s, lambda=seq(-2,2,length=21),family="yjPower")
  title(main="Modelo una covariable", col.main="darkred",
        cex.main=1.5)
