# Regresin dinmica lineal Multivariada

# Libreras
  suppressMessages(suppressWarnings(library(MARSS)))
  suppressMessages(suppressWarnings(library(imputeTS)))
  suppressMessages(suppressWarnings(library(ggplot2)))
  suppressMessages(suppressWarnings(library(forecast)))
  suppressMessages(suppressWarnings(library(GGally)))
  suppressMessages(suppressWarnings(library(changepoint))) 
  suppressMessages(suppressWarnings(library(strucchange)))
  suppressMessages(suppressWarnings(library(ggpmisc)))
  suppressMessages(suppressWarnings(library(ggfortify)))
  suppressMessages(suppressWarnings(library(lubridate)))
  suppressMessages(suppressWarnings(library(readxl)))
  suppressMessages(suppressWarnings(library(nortest)))

  #setwd("C:/Users/SIEC/Documents/TESIS/R/Insumos Tesis NMR/Datos Finales v3")
  setwd("C:\\Users\\DELL\\Desktop\\TESIS_NMR\\Insumos Tesis NMR\\Datos Finales v3")
  
  base <- read.csv("Datos_v3.csv")
  attach(base)
  fecha <- as.Date(as.character(fecha), "%d/%m/%Y")

# Diseo Grficas  
  theme_set(theme_classic())
  theme_update(plot.title = element_text(hjust = 0.5, size = 24, face = "bold"), 
               plot.subtitle=element_text(size=20, hjust=0.5),
               plot.caption = element_text(size = 18),
               axis.title=element_text(size=24),
               axis.text = element_text(size=22),
               legend.text=element_text(size=24), 
               legend.position="bottom",
               legend.box = "horizontal")#centrar el titulo en las grficas 
  
# Nmero de das de los datos 
  TT = length(fecha)

# Nmero de parmetros de regresin (B0, B1, B2)
  m = 3

# Variable respuesta Yt
  Yt = matrix(Yt.s,nrow=1)

# Covariables
  xt <- matrix(xt.s, nrow = 1)
  x2t <- matrix(x2t.s, nrow = 1)
  x3t <- matrix(x3t.s, nrow = 1)

#--- Modelo de regresin Dinmico ---

# Definicin de los parmetros de la ecuacin del sistema

# Matriz identidad; 3x3
  B = diag(m)                 
# Matriz de 0's; 3x1
  U = matrix(0,nrow=m,ncol=1) 
# Matriz de varianzas del sistema = 0's por ahora; 3x3
  Q = matrix(list(0),m,m) 
  # Regresin mltiple: diag(Wt)=[0,0,0] -> thetha_t=thetha 
    Q2 = matrix(list(0),m,m) 
# diag(Matriz de varianzas del sistema) = (q1,q2,q3); 3x3
  diag(Q) = c("q.beta0","q.beta1", "q.beta2" ) 
  # Regresin mltiple: diag(Wt)=[0,0,0] -> thetha_t=thetha  
    diag(Q2) = c(0,0,0) 

# Definicin de los parmetros de la ecuacin de observaciones

Z = array(NA, c(1,m,TT))   ## Z=Ft' vaca por ahora; NxMxT
Z[1,1,] = rep(1,TT)        ## 1's para el intercepto; Nx1
Z[1,2,] = xt               ## covariable xt; Nx1 
Z[1,3,] = x3t              ## covariable x3t; Nx1  
A = matrix(0)              ## escalar 0; 1x1
R = matrix("r")            ## escalar r; 1x1; 

# Valores iniciales 
  inits.list = list(x0=matrix(c(0, 0, 0), nrow=m))
# Lista de los parmetros que definen el modelo
  mod.list = list(B=B, U=U, Q=Q, Z=Z, A=A, R=R)
  # Regresin mltiple: diag(Wt)=[0,0,0] -> thetha_t=thetha 
    mod.list2 = list(B=B, U=U, Q=Q2, Z=Z, A=A, R=R)
# Ajuste del modelo multivariado dinmico lineal: Yt ~ xt + x3t
  dlm1 = MARSS(Yt, inits=inits.list, model=mod.list)
  # Regresin mltiple: diag(Wt)=[0,0,0] -> thetha_t=thetha
    dlm2 = MARSS(Yt, inits=inits.list, model=mod.list2)
    
#--- Grficos modelo de regresin dinmico lineal multivariado ---
    
# Intervalos de confianza para alpha, beta1 y beta2
  b0.u = dlm1$states[1,] + 1.96*dlm1$states.se[1,]
  b0.l = dlm1$states[1,] - 1.96*dlm1$states.se[1,]
  b1.u = dlm1$states[2,] + 1.96*dlm1$states.se[2,]
  b1.l = dlm1$states[2,] - 1.96*dlm1$states.se[2,]
  b2.u = dlm1$states[3,] + 1.96*dlm1$states.se[3,]
  b2.l = dlm1$states[3,] - 1.96*dlm1$states.se[3,]

# Grficas de los valores suavizados para alpha, beta1 y beta2 con 
# intervalos al 95% de confianza
  b0 <- data.frame(dlm1$states[1,], b0.u, b0.l)
  names(b0) <- c("b0.s", "b0.u", "b0,l")
  ggplot(b0, aes(x=fecha)) +
  geom_line(aes(y=b0.s, colour="blue"), size=1.5) +
  geom_line(aes(y=b0.u, colour="darkblue"), size=1.5) +
  geom_line(aes(y=b0.l, colour="darkblue"), size=1.5) +
  geom_hline(yintercept = 88.62, linetype=3, size=1.5) +
  scale_colour_manual("", values = c("blue", "darkblue", "darkblue"),
                      labels = c(expression(hat(beta)[0.2017]), "I.C.")) +
                      labs(title=" ", x=" ", y=expression(hat(beta)["0,t"]))+
                      theme(panel.background = element_rect(fill = "white", colour = "grey50"))

  
  b1 <- data.frame(dlm1$states[2,], b1.u, b1.l)
  names(b1) <- c("b1.s", "b1.u", "b1,l")
  ggplot(b1, aes(x=fecha)) +
  geom_line(aes(y=b1.s, colour="darkorange1"), size=1.5) +
  geom_line(aes(y=b1.u, colour="darkorange2"), size=1.5) +
  geom_line(aes(y=b1.l, colour="darkorange2"), size=1.5) +
  geom_hline(yintercept = 0, linetype=2, colour="red") +
  geom_hline(yintercept = 17.405, linetype=3, size=1.5) +  
  scale_colour_manual("", values = c("darkorange1", "darkorange2", "darkorange2"),
              labels = c(expression(hat(beta)[1.2017]), "I.C.")) +
              labs(title=" ", x=" ", y=expression(hat(beta)["1,t"])) +
              theme(panel.background = element_rect(fill = "white", colour = "grey50"))

  
  b2 <- data.frame(dlm1$states[3,], b2.u, b2.l)
  names(b2) <- c("b2.s", "b2.u", "b2,l")
  ggplot(b2, aes(x=fecha)) +
  geom_line(aes(y=b2.s, colour="darkgreen"), size=1.5) +
  geom_line(aes(y=b2.u, colour="forestgreen"), size=1.5) +
  geom_line(aes(y=b2.l, colour="forestgreen"), size=1.5) +
  geom_hline(yintercept = 0, linetype=2, colour="red") +
  geom_hline(yintercept = -6.642, linetype=3, size=1.5) +
           scale_colour_manual("", values = c("darkgreen", "forestgreen", "forestgreen"),
           labels = c(expression(hat(beta)[2.2017]), "I.C.")) +
           labs(title=" ", x=" ", y=expression(hat(beta)["2,t"]))+
           theme(panel.background = element_rect(fill = "white", colour = "grey50"))

  
#--- Prediccin del modelo dinmico lineal multivariado k=1 ---
  
  
# Salida del filtro de Kalman
  kf.out = MARSSkfss(dlm1)
  
# Prediccin de los parmetros de regresin; matriz de 3xT 
  eta = kf.out$xtt1
# Media de la distribucin predictiva un paso adelante para los estados 
# at=E(thetha_1:t-1|Yt-1)=Gt*mt-1
  fore.mean = vector()
  for(t in 1:TT) {
      fore.mean[t] = Z[,,t] %*% eta[,t,drop=F]
  }
  #--- Modelo de regresin mltiple------------
    kf.out2 = MARSSkfss(dlm2)
    eta2 = kf.out2$xtt1
    fore.mean2 = vector()
    for(t in 1:TT) {
      fore.mean2[t] = Z[,,t] %*% eta2[,t,drop=F]
    }
  #--------------------------------------------
    
# Varianza un paso adelante de los parmetros de regresin al tiempo t; 1x2xT
  Phi = kf.out$Vtt1
# Varianza de la ec. de observacin; matriz 1x1 
  R.est = coef(dlm1, type="matrix")$R
# Varianza de la distribucin predictiva
# Rt=Var(thetha_1:t-1|Yt-1)=Gt*Ct-1*Gt'+Wt
  fore.var = vector()
  for(t in 1:TT) {
      tZ = matrix(Z[,,t],m,1) ## transpose of Z
      fore.var[t] = Z[,,t] %*% Phi[,,t] %*% tZ + R.est
  }

    #--------------------------------------------  
      # Varianza un paso adelante de los parmetros de regresin al tiempo t; 1x2xT
        Phi2 = kf.out2$Vtt1
      # Varianza de la ec. de observacin; matriz 1x1 
        R.est2 = coef(dlm2, type="matrix")$R
      # Varianza de la distribucin predictiva
      # Rt=Var(thetha_1:t-1|Yt-1)=Gt*Ct-1*Gt'+Wt
        fore.var2 = vector()
        for(t in 1:TT) {
          tZ2 = matrix(Z[,,t],m,1) ## transpose of Z
          fore.var2[t] = Z[,,t] %*% Phi2[,,t] %*% tZ2 + R.est2
        }
    #---------------------------------------------

  
# Prediccin el vector de observaciones un paso hacia adelante
  fk_dlm1 = fore.mean
  fk_lower = fore.mean-2*sqrt(fore.var)
  fk_upper = fore.mean+2*sqrt(fore.var)
  forecast_dlm1 <- data.frame(fecha=fecha, Obs=Yt.s, fk1=fore.mean, low=fk_lower, upp=fk_upper)
  names(forecast_dlm1) <- c("fecha", "Yt.s", "fkdlm1", "lower", "upper")

  ggplot(forecast_dlm1, aes(x=fecha)) +
  geom_point(aes(y=Yt.s, colour="darkblue"), size=2.1) +
  geom_line(aes(y=fkdlm1, colour="darkgreen"), size=1.5) + 
  geom_line(aes(y=lower, colour="forestgreen"), size=1, linetype=1) +
  geom_line(aes(y=upper, colour="forestgreen"), size=1, linetype=1) +
  scale_colour_manual("", values = c("darkblue", "darkgreen", "forestgreen", 
                                     "forestgreen"), 
                          labels = c("Yt","Valores predichos modelo dinmico", 
                                     "Intervalos de 95% de confianza")) +
                          labs(title=" ", x=" ", y=expression(O[3](ppb)),
                          caption="\nDatos Red Automtica de Monitoreo Atmosfrico (RAMA)") +
                          theme(panel.background = element_rect(fill = "white", colour = "grey50"))

#--------------- Predicciones un paso adelante ----------------------
# Modelo de regresin mltiple vs Modelo Dinmico Lineal Multivariado
  
  fk_lower_est = fore.mean2-2*sqrt(fore.var2)
  fk_upper_est = fore.mean2+2*sqrt(fore.var2)
  
  forecast1 <- data.frame(fecha=fecha, Obs=Yt.s, fk1=fore.mean, fk2=fore.mean2, low=fk_lower, upp=fk_upper, 
                          low_est=fk_lower_est, upp_est=fk_upper_est)
  
  names(forecast1) <- c("fecha", "Yt.s", "fkdlm", "fkest", "lower", "upper", "low_est", "upp_est")

  ggplot(forecast1, aes(x=fecha)) +
  geom_point(aes(y=Yt.s, colour="blue"), size=1.2) + geom_point(aes(y=Yt.s), colour="blue", size=2.2, shape=15) +
  geom_line(aes(y=fkdlm, colour="darkgreen"), size=1) + geom_point(aes(y=fkdlm), colour="darkgreen", size=1.5) +
  geom_line(aes(y=lower, colour="forestgreen"), size=1.4, linetype=3) +
  geom_line(aes(y=upper, colour="forestgreen"), size=1.4, linetype=3) +
  geom_line(aes(y=fkest, colour="darkorange"), size=1, linetype=1) + geom_point(aes(y=fkest), colour="darkorange2", size=1.5) +
  geom_line(aes(y=low_est, colour="darkorange1"), size=1.4, linetype=3) +
  geom_line(aes(y=upp_est, colour="darkorange1"), size=1.4, linetype=3) +  
  scale_colour_manual("", values = c("blue", "darkgreen", "forestgreen","forestgreen","darkorange", "darkorange1", "darkorange1"),
                          labels = c("Yt ","fk MDLM", "I.C. MDLM", "fk modelo esttico", "I.C. modelo esttico")) +
                          labs(title=" ", 
                          caption="\nDatos Red Automtica de Monitoreo Atmosfrico (RAMA)", x=" ", y=expression(O[3](ppb)))+
  theme(panel.background = element_rect(fill = "white", colour = "grey50"))

  # MDLM: Conteo del nmero de observaciones que caen dentro del intervalo al 95% de confianza 
  cuenta = ifelse(forecast1$Yt.s>=forecast1$lower & forecast1$Yt.s<=forecast1$upper,1,0)
  sum(cuenta)
  sum(cuenta)/sum(!is.na(cuenta))
 
  
  # Modelo esttico: Conteo del nmero de observaciones que caen dentro del intervalo al 95% de confianza 
  cuentaMEST = ifelse(forecast1$Yt.s>=forecast1$low_est & forecast1$Yt.s<=forecast1$upp_est,1,0)
  sum(cuentaMEST)
  sum(cuentaMEST)/sum(!is.na(cuentaMEST))
  
#---------------- Validaciones del Modelo -----------------------------

# Modelo Dinmico Lineal Multivariado
  MAE <- mean(abs(fore.mean-t(Yt)))
  MSE1 <- mean((fore.mean-t(Yt))^2)
  MAPE1 <- mean(abs(fore.mean-t(Yt))/abs(t(Yt)))*100
  sqrt(sum((fore.mean - t(Yt))[-(1:5)]^2)/sum(diff(t(Yt)[-(1:4)])^2))
  #sqrt(sum((fore.mean - t(Yt))^2)/sum(diff(t(Yt))^2))

# Modelo de Regresin Lineal Multivariado
  MAE2 <- mean(abs(fore.mean2-t(Yt)))
  MSE2 <- mean((fore.mean2-t(Yt))^2)
  MAPE2 <- mean(abs(fore.mean2-t(Yt))/abs(t(Yt)))*100
  sqrt(sum((fore.mean2 - t(Yt))[-(1:5)]^2)/sum(diff(t(Yt)[-(1:4)])^2))
  #sqrt(sum((fore.mean2 - t(Yt))^2)/sum(diff(t(Yt))^2))

  
# Diagnsticos para los residuales del modelo 

  
# Errores de prediccion o innovaciones del modelo
  innov = kf.out$Innov

# ACF de los residuales
  acf(t(innov), main="")
  title(main="ACF de los Residuales", col.main="darkred")

# Ho: La media de la distribucin de las innovaciones es igual a cero 
# H1: La media de la distribucin de las innovaciones es distinta de cero
  
  t.test(t(innov))

# Ljung Box
  Box.test(t(innov), lag=20, type = "Ljung")
  p_values_LB <- sapply(1 : 15, function(i)
  Box.test(t(innov), lag=i, type="Ljung-Box")$p.value)

  plot(p_values_LB, type="p",  pch=19, ylim=c(0,1), xlim=c(0,16), lwd=1, col="blue", main=" ", xlab="Lag", ylab="p-valor")
  abline(h=0.05, col="darkred", lty=3, lwd=2)
  title(main="p-valores para el estadstico Ljung-Box", col.main="darkred")

# Histograma de los Residuales
  x=(-35:35)/10
  hist((t(innov)-mean(t(innov)))/sd(t(innov)),nclass=50,freq=F,
       border="blue", col="skyblue", xlab="Residuales", 
       ylab="Densidad", main=" ")
  title(main="Distribucin de los residuales", col.main="darkred")
  lines(x,dnorm(x), lwd=2)

# Q-Q plot de los residuales
  qqnorm(t(innov), main=" ", ylab="Cuantiles muestrales", xlab="Cuantiles teoricos")
  qqline(t(innov), col="blue", lwd=1.8, lty=1.3)
  title(main="Normal QQ-Plot", col.main="darkred")
  
# Prueba de Normalidad Anderson Darling
  ad.test(t(innov))
  t.test(t(innov))
  
  
#--- Prediccin de observaciones nuevas k=31 das hacia adelante ---

  
# Si bien la evalucin de ambos modelos y el prnostico de un paso
# hacia adelante de las observaciones sugieren que el modelo dinmico
# brinda un mejor ajuste para nuestros datos, mostraremos las
# predicciones del modelo k pasos hacia adelante.

# Pronstico de los estados (theta_t) k=31 pasos hacia adelante 

  # Valores verdaderos para las series O3, TMP y WSP
    base_2018 <- read.csv("base_2018.csv") 
    fecha_2018 <- as.Date(as.character(base_2018$fecha), "%d/%m/%Y")
    Yt_2018 <- base_2018[1:31,]

  # Estandarizar covariables
    xt_2018 <- (base_2018$TMP_max - mean(base_2018$TMP_max, na.rm=T))/sqrt(var(base_2018$TMP_max, na.rm=T))
    x3t_2018 <- (base_2018$WSP_max - mean(base_2018$WSP_max, na.rm=T))/sqrt(var(base_2018$WSP_max, na.rm=T))

  # Covariable temperatura
    xtnew = c(xt, xt_2018[1:31])
    xtnew = ts(xtnew, start = c(2017,1), frequency = 365)
  # Covariable WSP  
    x3tnew = c(x3t, x3t_2018[1:31])
    x3tnew <- ts(x3tnew, start = c(2017,1), frequency = 365)

  # Nmero de obs. nuevas a predecir   
    TTnew = 31   
  # Vector de NA's de longitud 31 
    Ytnew = rep(NA,TTnew)
  # Matriz de observaciones concatenada con los 31 NA's  
    Yt3 = matrix(c(Yt,Ytnew),nrow=1)
  # Matriz de dimensiones NxMxT; vaca por ahora  
    Z3 = array(NA, c(1,m,(TT+TTnew)))   
  # Matriz de 1's para el intercepto;  Nx1
    Z3[1,1,] = rep(1,(TT+TTnew))        
  # Covariables temperatura; Nx1
    Z3[1,2,] = xtnew 
  # Covariables velocidad del viento; Nx1  
    Z3[1,3,] = x3tnew             

  # Definicin y ajuste del modelo dinmico lineal
    mod.list3 = list(B=B, U=U, Q=Q, Z=Z3, A=A, R=R)
    dlm3 = MARSS(Yt3, inits=inits.list, model=mod.list3)
    #--- Definicin y ajuste del modelo de regresin mltiple ---
    mod.list3_est = list(B=B, U=U, Q=Q2, Z=Z3, A=A, R=R)
    dlm3_est = MARSS(Yt3, inits=inits.list, model=mod.list3_est)
    
# Grficos Modelo Predictivo k=31 pasos hacia adelante
    
  # Prediccin de Estados MDLM 
    
# Intervalos de confianza para alpha, beta1 y beta2
  b0.u.new = dlm3$states[1,] + 1.96*dlm3$states.se[1,]
  b0.l.new = dlm3$states[1,] - 1.96*dlm3$states.se[1,]  
  b1.u.new = dlm3$states[2,] + 1.96*dlm3$states.se[2,]
  b1.l.new = dlm3$states[2,] - 1.96*dlm3$states.se[2,]
  b2.u.new = dlm3$states[3,] + 1.96*dlm3$states.se[3,]
  b2.l.new = dlm3$states[3,] - 1.96*dlm3$states.se[3,]

# Valores suavizados para alpha, beta1 y beta2 e intervalos de confianza
  fecha.new = c(fecha, fecha_2018[1:31])

  b0.new <- data.frame(c(dlm3$states[1,1:365], rep(NA,31)), c(rep(NA,365),dlm3$states[1,366:dim(dlm3$states)[2]]),
                       b0.u.new, b0.l.new)
  names(b0.new) <- c("b0.s1", "b0.s2", "b0.u", "b0.l")
  ggplot(b0.new, aes(x=fecha.new)) +
  geom_line(aes(y=b0.s1, colour="blue"), size=1.5) +
  geom_line(aes(y=b0.s2, colour="cornflowerblue"), size=1.5) +
  geom_line(aes(y=b0.u, colour="darkblue"), size=1.5) +
  geom_line(aes(y=b0.l, colour="darkblue"), size=1.5) +
  #geom_vline(aes(xintercept=fecha.new[365]), size=1.5, linetype=2, color="darkred")+
  scale_colour_manual("", values = c("blue", "hotpink", "darkblue", "darkblue"), 
  labels = c(expression(hat(beta)[0.2017]), expression(hat(beta)[0.2018]), "I.C.")) +
  labs(title=" ", x=" ", y=expression(hat(beta)["0,t"]))+
  theme(panel.background = element_rect(fill = "white", colour = "grey50"))

  b1.new <- data.frame(c(dlm3$states[2,1:365], rep(NA,31)), c(rep(NA,365),dlm3$states[2,366:dim(dlm3$states)[2]]),
                       b1.u.new, b1.l.new)
  names(b1.new) <- c("b1.s1", "b1.s2", "b1.u", "b1.l")
  ggplot(b1.new, aes(x=fecha.new)) +
  geom_line(aes(y=b1.s1, colour="darkorange1"), size=1.5) +
  geom_line(aes(y=b1.s2, colour="cornflowerblue"), size=1.5) +
  geom_line(aes(y=b1.u, colour="darkorange2"), size=1.5) +
  geom_line(aes(y=b1.l, colour="darkorange2"), size=1.5) +
  #geom_vline(aes(xintercept=fecha.new[365]), size=1.5, linetype=2, color="darkred")+
  geom_hline(yintercept = 0, linetype=2) +
  scale_colour_manual("", values = c("hotpink", "darkorange1", "darkorange2", "darkorange2"), 
  labels = c(expression(hat(beta)[1.2017]), expression(hat(beta)[1.2018]), "I.C.")) +
  labs(title=" ", x=" ", y=expression(hat(beta)["1,t"]))+
  theme(panel.background = element_rect(fill = "white", colour = "grey50"))

  b2.new <- data.frame(c(dlm3$states[3,1:365], rep(NA,31)), c(rep(NA,365),dlm3$states[3,366:dim(dlm3$states)[2]]),
                       b2.u.new, b2.l.new)
  names(b2.new) <- c("b2.s1", "b2.s2", "b2.u", "b2.l")
  ggplot(b2.new, aes(x=fecha.new)) +
  geom_line(aes(y=b2.s1, colour="darkgreen"), size=1.5) +
  geom_line(aes(y=b2.s2, colour="hotpink"), size=1.5) +
  geom_line(aes(y=b2.u, colour="forestgreen"), size=1.5) +
  geom_line(aes(y=b2.l, colour="forestgreen"), size=1.5) +
  #geom_vline(aes(xintercept=fecha.new[365]), size=1.5, linetype=2, color="darkred")+
  geom_hline(yintercept = 0, linetype=2) +
  scale_colour_manual("", values = c( "darkgreen", "forestgreen", "hotpink", "forestgreen"), 
                      labels = c(expression(hat(beta)[2.2017]), expression(hat(beta)[2.2018]), "I.C.")) +
  labs(title=" ", x=" ", y=expression(hat(beta)["2,t"]))+
  theme(panel.background = element_rect(fill = "white", colour = "grey50"))

  #Prnostico MDLM k=31
 
  # Salida del filtro de Kalman
    kf.out3 = MARSSkfss(dlm3) 
  # Prediccin de los parmetros de regresin; matriz de 3xT 
    at3 = kf.out3$xtt1
  # Media de la distribucin predictiva un paso adelante para los estados 
    ft3 = vector()
    for(t in 1:(TT+TTnew)) {
        ft3[t] = Z3[,,t] %*% at3[,t,drop=F]
    }
    
    #--- Prnostico modelo de regresin mltiple ---
     kf.out3_est = MARSSkfss(dlm3_est)  
     at3_est = kf.out3_est$xtt1
     ft3_est = vector()
     for(t in 1:(TT+TTnew)) {
       ft3_est[t] = Z3[,,t] %*% at3_est[,t,drop=F]
     }
      
  # Varianza un paso adelante de los parmetros de regresin al tiempo t; 1x2xT
    Rt3 = kf.out3$Vtt1
  # Varianza de la ec. de observacin; matriz 1x1 
    Vt3 = coef(dlm3, type="matrix")$R
  # Varianza de la distribucin predictiva
    Qt3 = vector()
    for(t in 1:(TT+TTnew)) {
        tZ3 = matrix(Z3[,,t],m,1) #Matriz transpuesta de Z
        Qt3[t] = Z3[,,t] %*% Rt3[,,t] %*% tZ3 + Vt3
    }
    
    #--- Prnostico modelo de regresin mltiple ---
    Rt3_est = kf.out3_est$Vtt1
    Vt3_est = coef(dlm3_est, type="matrix")$R
    Qt3_est=vector()
    for(t in 1:(TT+TTnew)) {
      tZ3_est = matrix(Z3[,,t],m,1) #Matriz transpuesta de Z
      Qt3_est[t] = Z3[,,t] %*% Rt3_est[,,t] %*% tZ3_est + Vt3_est
    }
    

  # Valores predichos MDLM
    fk_dlm3 = ft3
    #--- Valores predichos modelo de regresin mltiple
      fk_dlm3_est = ft3_est
  # Intervalos de confianza 
    fk_lower3 = ft3-1.96*sqrt(Qt3)
    fk_upper3 = ft3+1.96*sqrt(Qt3)
    # --- Intervalos de confianza modelo de regresin mltiple
      fk_lower3_est = ft3_est-1.96*sqrt(Qt3_est)
      fk_upper3_est = ft3_est+1.96*sqrt(Qt3_est)
 
  # Serie de datos observado (reales)  
    Yt.s3 <- c(Yt.s, rep(NA,31))
    Yt_nuevo <- c(rep(NA,365), base_2018$PED_max[1:31])
    Yt_comp <- c(Yt.s, base_2018$PED_max[1:31])
    fk_dlm3_1 <- c(fk_dlm3[1:365], rep(NA, 31))
    fk_dlm3_2 <- c(rep(NA, 365), fk_dlm3[366:length(fk_dlm3)])
    
    forecast_dlm3 <- data.frame(fecha=fecha.new, Obs=Yt.s3, new_real=Yt_nuevo , fk1=fk_dlm3_1,
                                fk1=fk_dlm3_2, low=fk_lower3, upp=fk_upper3)
    names(forecast_dlm3) <- c("fecha.new", "Yt3", "new_real", "fkdlm3_1", "fkdlm3_2", "lower", "upper")

    ggplot(forecast_dlm3, aes(x=fecha.new)) +
    geom_point(aes(y=Yt3, colour="cornflowerblue"), size=2.2) +
    geom_point(aes(y=new_real, colour="darkblue"), size=2.2) +  
    geom_line(aes(y=fkdlm3_1, colour="darkgreen"), size=1.5) + 
    geom_line(aes(y=fkdlm3_2, colour="indianred1"), size=1.5) +   
    geom_line(aes(y=lower, colour="forestgreen"), size=1, linetype=1) +
    geom_line(aes(y=upper, colour="forestgreen"), size=1, linetype=1) +
    #geom_vline(aes(xintercept=fecha.new[365]), size=1.5, linetype=2, color="darkred") +
    scale_colour_manual("", values = c("darkblue", "cornflowerblue", "darkgreen","forestgreen", "indianred1", "forestgreen"), 
                      labels = c(expression(Y[t.2017]), expression(Y[t.2018]), expression(f[t.2017]),"I.C.",  expression(f[t.2018]))) +
    labs(title="Prediccin de las concentraciones de ozono k=31 das", 
         subtitle="Serie de las concentracines de Ozono Mximas Diarias del Promedio Mvil de 8 hrs\nEstacin Pedregal (PED), Delegacin Alvaro Obregn, CDMX (2017)\n",
         caption="\nDatos Red Automtica de Monitoreo Atmosfrico (RAMA)", 
         x=" ", y=expression(O[3](ppb))) +
    theme(panel.background = element_rect(fill = "white", colour = "grey50"))

    #--- Prnostico MDLM vs modelo de regresin mltiple
    
      fk_dlm3_1_est <- c(fk_dlm3_est[1:365], rep(NA, 31))
      fk_dlm3_2_est <- c(rep(NA, 365), fk_dlm3_est[366:length(fk_dlm3_est)])
    
      forecast_dlm3_est <- data.frame(fecha=fecha.new, Obs=Yt.s3, new_real=Yt_nuevo , 
                                      fk_1=fk_dlm3_1,fk_2=fk_dlm3_2, low=fk_lower3, upp=fk_upper3,
                                      fk_1_est=fk_dlm3_1_est, fk_2_est=fk_dlm3_2_est, 
                                      low_est=fk_lower3_est, upp_est=fk_upper3_est)
                                      
      names(forecast_dlm3_est) <- c("fecha.new", "Yt3", "new_real", 
                                "fkdlm3_1", "fkdlm3_2", "lower", "upper",
                                "fkdlm3_1_est", "fkdlm3_2_est", "lower_est", "upper_est")
  
      
    ggplot(forecast_dlm3_est, aes(x=fecha.new)) +
      geom_point(aes(y=Yt3, colour="darkblue"), size=2.2) + geom_point(aes(y=Yt3), colour="darkblue", size=2.2, shape=15) +
      geom_point(aes(y=new_real, colour="cornflowerblue"), size=2.2) + geom_point(aes(y=new_real), colour="cornflowerblue", size=2.2, shape=15) +
      geom_line(aes(y=fkdlm3_1, colour="darkgreen"), size=1.5) + geom_point(aes(y=fkdlm3_1), colour="darkgreen", size=2.2) +
      geom_line(aes(y=fkdlm3_2, colour="indianred1"), size=1.5) +
      geom_line(aes(y=lower, colour="forestgreen"), size=1, linetype=3) +
      geom_line(aes(y=upper, colour="forestgreen"), size=1, linetype=3) +
      geom_line(aes(y=fkdlm3_1_est, colour="darkorange"), size=1.5) +
      geom_line(aes(y=fkdlm3_2_est, colour="violet"), size=1.5) +
      geom_line(aes(y=lower_est, colour="darkorange1"), size=1, linetype=3) +
      geom_line(aes(y=upper_est, colour="darkorange1"), size=1, linetype=3) +
      #geom_vline(aes(xintercept=fecha.new[365]), size=1.5, linetype=2, color="darkred") +
      scale_colour_manual("", #values = c("darkblue", "cornflowerblue", "darkgreen", 
                              #           "forestgreen", "indianred1", "forestgreen"), 
                          values = c("darkblue", "cornflowerblue", "darkgreen", "darkorange",
                                     "forestgreen", "darkorange1",
                                     "indianred1", "violet","darkorange1", "darkorange1"),
                          labels = c(expression(Y[t.orig.2017]), expression(Y[t.orig.2018]), expression(f[t.2017.dinmico]), 
                                     expression(f[t.2017.esttico]),"I.C. Modelo dinmico",  "I.C. Modelo esttico", 
                                     expression(f[t.2018.dinmico]), expression(f[t.2018.esttico]))) +
      labs(title="Prediccin de las concentraciones de ozono k=31 das", 
           subtitle="Serie de las concentracines de Ozono Mximas Diarias del Promedio Mvil de 8 hrs\nEstacin Pedregal (PED), Delegacin Alvaro Obregn, CDMX (2017)\n",
           caption="\nDatos Red Automtica de Monitoreo Atmosfrico (RAMA)", 
           x=" ", y=expression(O[3](ppb))) +
      theme(panel.background = element_rect(fill = "white", colour = "grey50"))
    
    #---------------- Evaluacin del Modelo k=31 -----------------------------
    
    # Modelo Dinmico Lineal Multivariado
    MAE3 <- mean(abs(fk_dlm3-Yt_comp))
    MSE3 <- mean((fk_dlm3-Yt_comp)^2)
    MAPE3 <- mean(abs(fk_dlm3-Yt_comp)/abs(Yt_comp))*100
    sqrt(sum((fk_dlm3 - Yt_comp)[-(1:5)]^2)/sum(diff(Yt_comp[-(1:4)])^2))
    #sqrt(sum((fore.mean - t(Yt))^2)/sum(diff(t(Yt))^2))
    
    # Modelo de Regresin Lineal Multivariado
    MAE3_est <- mean(abs(fk_dlm3_est-Yt_comp))
    MSE3_est <- mean((fk_dlm3_est-Yt_comp)^2)
    MAPE3_est <- mean(abs(fk_dlm3_est-Yt_comp)/abs(Yt_comp))*100
    sqrt(sum((fore.mean2 - t(Yt))[-(1:5)]^2)/sum(diff(t(Yt)[-(1:4)])^2))
    #sqrt(sum((fore.mean2 - t(Yt))^2)/sum(diff(t(Yt))^2))
    
    
     
    # MDLM: Conteo del nmero de observaciones que caen dentro del intervalo al 95% de confianza 
    cuenta3 = ifelse(Yt_comp<fk_upper3 & Yt_comp>fk_lower3,1,0)
    sum(cuenta3)
    sum(cuenta3)/sum(!is.na(cuenta3))
    
      # Modelo esttico: Conteo del nmero de observaciones que caen dentro del intervalo al 95% de confianza 
      cuenta3_est = ifelse(Yt_comp<fk_upper3_est & Yt_comp>fk_lower3_est,1,0)
      sum(cuenta3_est)
      sum(cuenta3_est)/sum(!is.na(cuenta3_est))
    